'''
Created on Sep 18, 2018

@author: Zuhair Hammadi
'''
from odoo import models, api, fields
import datetime
from collections import defaultdict
import re
import uuid

def normilize(*names):
    name = ' '.join(filter(lambda i : isinstance(i, str),names))
    return re.sub('\W',' ', name.lower()).strip().replace(' ', '_')     

class Base(models.AbstractModel):
    _inherit ='base'
    
    @api.model
    def _base(self, model = 'base'):
        return type(self.env[model])
    
    @api.multi
    def _read_field(self, field):
        """
        :param field name
        :return: dictionary mapping record id with field value
        """
        res = self.read([field])
        res = {record['id'] : record[field] for record in res}
        return res
    
    @api.multi
    def _create_external_id(self):
        ref = self.get_external_id().get(self.id)        
        if not ref:
            IrModelData = self.env['ir.model.data']
            vals = {'module' : '_',
                    'name' : normilize(self._name, 'code' in self and self['code'] and len(self['code']) < 20 or '', self.display_name)
                    }
            while IrModelData.search(self._dict_to_domain(vals)):
                vals['name'] = '%s_%s' % (vals['name'], uuid.uuid4().hex[:6])            
            vals.update({
                'model' : self._name,
                'res_id' : self.id
                })
            ref=IrModelData.create(vals).complete_name     
        return ref       
        
    @api.model
    def _isinstance(self, model):
        return isinstance(self, type(self.env[model]))
    
    @api.model
    def _get_sql_value(self, sql, para = ()):
        self._cr.execute(sql, para)
        res=self._cr.fetchone()
        res= res and res[0] or False
        if isinstance(res, datetime.datetime):
            return fields.Datetime.to_string(res)
        if isinstance(res, datetime.date):
            return fields.Date.to_string(res)
        return res     
    
    @api.model
    def _dict_to_domain(self, vals):
        domain = []
        for key, value in vals.items():
            if isinstance(value, (dict,list,tuple )):
                value = str(value)
            domain.append((key, '=', value))
        return domain
    
    @api.multi
    def get_title(self):        
        return '%s | %s' % (self.env['ir.model']._get(self._name).name, self.display_name)    
    
    @api.multi
    def get_form_url(self):
        return '/web#id=%d&model=%s&view_type=form' % (self.id, self._name)
    
    def with_sudo(self):
        return self.with_context(_sudo = (1,))
                          
    @api.model
    def _grant_access(self, operation, *models):
        if operation=='all':
            self.env._skip_access_check= True
            return self   
        skip_access = getattr(self.env, '_skip_access_check', defaultdict(set))
        if skip_access is True:
            return self
        for model in models:
            skip_access[operation].add(model)
        self.env._skip_access_check= skip_access
        return self    
    
    @api.model
    def _revoke_access(self):
        if hasattr(self.env, '_skip_access_check'):
            del self.env._skip_access_check
        return self                
    
    @api.model
    def _skip_access_check(self, operation):
        if self._context.get("_sudo") == (1,):
            return True
        skip_access = getattr(self.env, '_skip_access_check', False)
        if not skip_access:
            return False
        if skip_access is True:
            return True        
        return self._name in skip_access.get(operation, [])
        
    @api.model
    def check_access_rights(self, operation, raise_exception=True):
        if self._skip_access_check(operation):
            return True               
        return super(Base, self).check_access_rights(operation, raise_exception=raise_exception)    
    
    @api.multi
    def check_access_rule(self, operation):
        if self._skip_access_check(operation):
            return True               
        return super(Base, self).check_access_rule(operation)
    
    @api.model
    def check_field_access_rights(self, operation, fields):
        if self._skip_access_check(operation):
            return fields or list(self._fields)         
        return super(Base, self).check_field_access_rights(operation, fields)

    @api.model
    def _apply_ir_rules(self, query, mode='read'):
        if self._skip_access_check(mode):
            return
        return super(Base, self)._apply_ir_rules(query, mode=mode)                
                
    @api.multi
    def onchange(self, values, field_name, field_onchange):
        self = self.with_context(onchange_field_name = field_name)
        return super(Base, self).onchange(values, field_name, field_onchange)     
        